;
;               U S G A R D   S C R E E N   C A P T U R E
;                               (usgcap, usc)
;                           written by Andreas Ess
;
;This is a cool demo TSR to make screenshots of normal and grayscale games.
;
#include "usgard.h"
#include "ugapplib.h"
#include "gray4lib.h"

.org 0

.db     "USC! 0.9 by AE",0
.db	UGAPPLIB,0

Start:
	call    CLEARLCD
	ld      hl, VIDEO_MEM
	ld      b, 112
FillLoop:
	ld      (hl), 255
	inc     hl
	djnz    FillLoop

	ld      hl, $0028
	ld      (CURSOR_X), hl

	set     3, (iy+5)
	ld      hl, (PROGRAM_ADDR)      ;PROGRAM_ADDR = Title
	call    D_ZM_STR
	res     3, (iy+5)

;Check if TSR is already installed using CHECK_APPEND.
; HL -> code to check(we only check the unmodified one)
; BC -> length of code.
	ld      hl, &StartCompare        ;(since code is the same from here)
	ld      bc, RoutineEnd-StartCompare
	LIB_CALL_(UGAPPLIB,CHECK_APPEND)
	jr      z, InstallTSR
	jp      nc, &NotAtTheEnd
	jr      nz, RemoveTSR           ;TSR already installed?

InstallTSR:

	ld      hl, $0001
	ld      (CURSOR_ROW), hl
	ld      hl, &InstTxt
	call    D_ZT_STR
Choice:
	halt
	call    GET_KEY
	cp      K_EXIT
	ret     z
	cp      K_F1
	jr      z, Normal
	cp      K_F2
	jr      nz, Choice
Grayscale:
	ld      hl, &Capture
	ld      bc, 2050
	push    hl
	call    VAR_GET
	pop     hl
	jr      c, MakeVar
	ld      hl, &Number
	inc     (hl)
	jr      Grayscale
MakeVar:
	ld      a, $0C
	call    VAR_NEW
	ld      (hl), 0
	inc     hl
	ld      (hl), $7E
	inc     hl
	ret     c                       ;return if error
        ld	a, $37			;SCF
	jr	Normal_1
Normal:
	ld	a, $B7			;OR A = reset Carry
Normal_1:
	ld      (&GrayorBW), a
	ld      hl, &TSRRoutine         ;...or use the & operator to add (PROGRAM_ADDR)
	ld      bc, RoutineEnd-TSRRoutine
	LIB_CALL_(UGAPPLIB,APPEND)      ;use UGAPPLIB to anchor code to Usgard.
	push    hl                      ;HL = pointer to actual address of anchored code
	ld      de, LdCap-TSRRoutine+2  ;Go to actual address of 'ld ix, Capture'
	push    hl
	add     hl, de
	ex      de, hl
	pop     hl                      ;DE = location of 'ld ix, ...'
	ld      bc, Capture-TSRRoutine
	add     hl, bc                  ;HL = location of appended Capture
	ex      de, hl
	ld      (hl), e
	inc     hl
	ld      (hl), d
	pop     hl
	call    INT_INSTALL             ;Install the interrupt!

	ld      hl, &Installed

WriteText:
	ld      de, $0003
	ld      (CURSOR_ROW), de

	call    D_ZT_STR

	call    OTH_PAUSE
	ret
RemoveTSR:
	call    INT_REMOVE

	ld      bc, RoutineEnd-TSRRoutine
	LIB_CALL_(UGAPPLIB,UNAPPEND)

	ld      hl, &Removed
	jr      WriteText
NotAtTheEnd:
	ld      hl, &NotAtEnd
	jr      WriteText

TSRRoutine:
	in      a, (3)                  ;Check ON
	bit     3, a
	ret     nz
	ld      a, %10111111            ;Check F1
	out     (1), a
	in      a, (1)
	bit     4, a
	ret     nz
	push    ix
	jr      LdCap                   ;Skip data
Capture:                                ;you mustn't put modifying data
	.db     "USC_"                  ;at the end - this leads to conflicts
Number:                                 ;with CHECKAPPEND
	.db     "0",0
LdCap:
	ld      ix, Capture
	push    ix
	pop     hl
GrayorBW:
	scf     
	jr      c, GrayscaleS
StartCompare:
	inc     (ix+4)
	ld      bc, 1026
	ld      a, $0C
	call    VAR_NEW
	jr      c, Error
	ld      (hl), 0
	inc     hl
	ld      (hl), $7D
	inc     hl
	ex      de, hl
	ld      hl, VIDEO_MEM
	ld      bc, 1024
	ldir
	jr      WaitON
GrayscaleS:
	call    VAR_GET
	inc     hl
	inc     hl
	ex      de, hl
	ld      hl, (Page1Addr)
	ld      bc, 1024
	ldir
	ld      hl, (Page2Addr)
	ld      bc, 1024
	ldir
	pop     hl
Error:
WaitON:
	in      a, (3)
	bit     3, a
	jr      z, WaitON
	pop     ix
	ret
RoutineEnd:

Installed:
	;012345678901234567890
 .db    "Activate = ON + F1",0
InstTxt:
 .db    "[F1] B/W screenshots "
 .db    "[F2] 1 GR4 screenshot",0
Removed:
 .db    "Removed",0
NotAtEnd:
 .db    "Not at the end!",0

.end
